@@ -1,10 +1,9 @@
 ;;;; banner.scm
 
 
-(define-constant +product+ "CHICKEN")
-
 (define-constant +banner+ #<<EOF
-(c) 2008-2017, The CHICKEN Team
+CHICKEN
+(c) 2008-2019, The CHICKEN Team
 (c) 2000-2007, Felix L. Winkelmann
 
 EOF
