@@ -1,6 +1,6 @@
 ;;;; build-version.scm
 ;
-; Copyright (c) 2011-2017, The CHICKEN Team
+; Copyright (c) 2011-2019, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -27,21 +27,20 @@
 (declare
  (unit build-version))
 
-(foreign-declare "#include \"buildtag.h\"")
-
 ;; (read-version filename): Read line from FILENAME and return
 ;; as a string; return #f if non-existent file or blank line.
 (define-syntax read-version
   (er-macro-transformer
    (lambda (x r c)
      (let ((fn (cadr x)))
-       (and (file-exists? fn)
-	    (let ((ver (with-input-from-file (cadr x) read-line)))
-	      (if (or (eof-object? ver) (string=? ver ""))
-		  #f
-		  ver)))))))
+       (and (##sys#file-exists? fn #t #f #f)
+	    (call-with-input-file (cadr x)
+	     (lambda (p)
+	       (let ((ver ((##sys#slot (##sys#slot p 2) 8) p 256))) ; read-line
+		 (if (or (eof-object? ver) (string=? ver ""))
+		     #f
+		     ver)))))))))
 
-(define (##sys#build-tag)   (foreign-value "C_BUILD_TAG" c-string))
 (define ##sys#build-id      (read-version "buildid"))
 (define ##sys#build-branch  (read-version "buildbranch"))
 (define ##sys#build-version (read-version "buildversion"))
