@@ -1,6 +1,6 @@
 ;;;; chicken.scm - The CHICKEN Scheme compiler (loader/main-module)
 ;
-; Copyright (c) 2008-2017, The CHICKEN Team
+; Copyright (c) 2008-2019, The CHICKEN Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -27,24 +27,35 @@
 
 (declare
   (uses chicken-syntax chicken-ffi-syntax 
-	srfi-1 srfi-4 utils files extras data-structures srfi-69
+	srfi-4 extras data-structures
 	lolevel ; unused, but loaded to make foldable bindings available
 	support compiler optimizer lfa2 compiler-syntax scrutinizer
-	driver platform backend))
+	batch-driver c-platform c-backend user-pass))
 
+(module chicken.compiler.chicken ()
+
+(import scheme
+	chicken.base
+	chicken.compiler.batch-driver
+	chicken.compiler.c-platform
+	chicken.compiler.support
+	chicken.compiler.user-pass
+	chicken.fixnum
+	chicken.process-context
+	chicken.string)
 
-(include "compiler-namespace")
 (include "tweaks")
-
+(include "mini-srfi-1.scm")
 
 ;;; Prefix argument list with default options:
 
 (define compiler-arguments
-  (append
-   (remove 
-    (lambda (x) (string=? x ""))
-    (string-split (or (get-environment-variable "CHICKEN_OPTIONS") "")))
-   (cdr (argv))))
+  (let ((args (cdr (argv))))
+    (if (null? args)
+        '()
+        (append (take args 1) ; Leave source filename argument first.
+                (string-split (or (get-environment-variable "CHICKEN_OPTIONS") ""))
+                (drop args 1)))))
 
 
 ;;; Process command-line options:
@@ -72,6 +83,7 @@
 ;;; Run compiler with command-line options:
 
 (receive (filename options) ((or (user-options-pass) process-command-line) compiler-arguments)
+  ;; TODO: Perhaps option parsing should be moved to batch-driver?
   (let loop ((os options))
     (unless (null? os)
       (let ((o (car os))
@@ -85,14 +97,13 @@
 		   ((1)
 		    (set! options
 		      (cons* 'optimize-leaf-routines
-			     ;XXX 'lfa2 
 			     options)) )
 		   ((2)
 		    (set! options 
 		      (cons* 'optimize-leaf-routines
 			     'inline
 			     ;XXX 'clustering
-			     ;XXX 'lfa2
+			     'lfa2
 			     options)) ) 
 		   ((3)
 		    (set! options
@@ -101,7 +112,7 @@
 			     'inline-global
 			     'local
 			     ;XXX 'clustering
-			     ;XXX 'lfa2
+			     'lfa2
 			     'specialize
 			     options) ) )
 		   ((4)
@@ -111,7 +122,7 @@
 			     'inline-global
 			     'specialize
 			     ;XXX 'clustering
-			     ;XXX 'lfa2
+			     'lfa2
 			     'local 'unsafe
 			     options) ) )
 		   (else
@@ -133,22 +144,22 @@
 			       options) ) ) ) )
 		 (loop (cdr rest)) ) )
 	      ((eq? 'debug-level o)
-	       (let ((level (string->number (car rest))))
-		 (case level
-		   ((0) (set! options (cons* 'no-lambda-info 'no-trace options)))
-		   ((1) (set! options (cons 'no-trace options)))
-		   ((2) (set! options (cons 'scrutinize options)))
-		   (else (set! options (cons* 'scrutinize 'debug-info options))))
-		 (loop (cdr rest)) ) )
+	       (case (string->number (car rest))
+		 ((0) (set! options (cons* 'no-lambda-info 'no-trace options)))
+		 ((1) (set! options (cons 'no-trace options)))
+		 ((2)) ; default behaviour
+		 ((3) (set! options (cons 'debug-info options)))
+		 (else (quit-compiling "invalid debug level: ~a" (car rest))))
+	       (loop (cdr rest)))
 	      ((memq o valid-compiler-options) (loop rest))
 	      ((memq o valid-compiler-options-with-argument)
 	       (if (pair? rest)
 		   (loop (cdr rest))
-		   (quit "missing argument to `-~s' option" o) ) )
+		   (quit-compiling "missing argument to `-~s' option" o) ) )
 	      (else
 	       (warning 
 		"invalid compiler option (ignored)" 
 		(if (string? o) o (conc "-" o)) )
 	       (loop rest) ) ) ) ) )
-  (apply compile-source-file filename options)
-  (exit) )
+  (apply compile-source-file filename compiler-arguments options)
+  (exit)))
