@@ -1,6 +1,6 @@
 ;;;; common-declarations.scm - settings for core libraries
 ;
-; Copyright (c) 2011-2017, The CHICKEN Team
+; Copyright (c) 2011-2019, The CHICKEN Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -31,11 +31,9 @@
  (debugbuild
   (define-syntax d
     (syntax-rules ()
-      ((_ arg1)
-       (when (##sys#fudge 13) (pp arg1))) ; debug-mode
-      ((_ arg1 more ...)
-       (when (##sys#fudge 13)
-	 (print arg1 more ...))))))
+      ((_ arg ...)
+       (when (##sys#debug-mode?)
+	 (print arg ...))))))
  (else
   (begin
     (declare
