@@ -1,6 +1,6 @@
 /* dbg-stub.c - Client-side interface, lowlevel part
 ;
-; Copyright (c) 2008-2017, The CHICKEN Team
+; Copyright (c) 2008-2019, The CHICKEN Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -53,7 +53,7 @@ static WSADATA wsa;
 #endif
 
 
-#define C_DEBUG_PROTOCOL_VERSION         0
+#define C_DEBUG_PROTOCOL_VERSION         1
 
 #define C_DEBUG_REPLY_UNUSED             0
 #define C_DEBUG_REPLY_SETMASK            1
@@ -75,7 +75,11 @@ static WSADATA wsa;
 
 #ifdef C_SIXTY_FOUR
 # define C_HEADER_BITS_SHIFT      56
-# define UWORD_COUNT_FORMAT_STRING     "%lu"
+# ifdef C_LLP
+#  define UWORD_COUNT_FORMAT_STRING     "%llu"
+# else
+#  define UWORD_COUNT_FORMAT_STRING     "%lu"
+# endif
 #else
 # define C_HEADER_BITS_SHIFT      24
 # define UWORD_COUNT_FORMAT_STRING     "%u"
@@ -114,7 +118,7 @@ static volatile int interrupted = 0;
 static int dbg_info_count = 0;
 
 
-static C_word debug_event_hook(C_DEBUG_INFO *cell, C_word c, C_word *av, C_char *cloc, int cln);
+static C_word debug_event_hook(C_DEBUG_INFO *cell, C_word c, C_word *av, C_char *cloc);
 
 
 void
@@ -155,7 +159,7 @@ socket_read()
       if(*(input_buffer_top++) == '\n') {
         *ptr = '\0';
         --input_buffer_len;
-        return 0;
+        return 1;
       }
 
       if(++off >= RW_BUFFER_SIZE) return -1; /* read-buffer overflow */
@@ -167,6 +171,8 @@ socket_read()
 
     if(n == SOCKET_ERROR) return -1; /* read failed */
 
+    if(n == 0) return 0; /* client disconnect */
+
     input_buffer_len = n;
     input_buffer_top = input_buffer;
   }
@@ -234,7 +240,7 @@ enable_debug_info(int n, int f)
     C_DEBUG_INFO *dinfo;
 
     for(dip = dbg_info_list; dip != NULL; dip = dip->next) {
-        for(dinfo = dip->info; dinfo->loc != NULL; ++dinfo) {
+        for(dinfo = dip->info; dinfo->event; ++dinfo) {
             if(i++ == n) {
                 dinfo->enabled = f;
                 return;
@@ -247,7 +253,7 @@ enable_debug_info(int n, int f)
 
 
 static void
-send_string(char *str)
+send_string(C_char *str)
 {
   /* fprintf(stderr, "<SENT: %s>\n", str); */
   C_fflush(stderr);
@@ -256,9 +262,18 @@ send_string(char *str)
     terminate("write failed");
 }
 
+static void
+send_string_value(C_char *str) {
+  if (str == 0 || *str == 0)
+    send_string(" #f");
+  else {
+    C_snprintf(rw_buffer, sizeof(rw_buffer), " \"%s\"", str);
+    send_string(rw_buffer);
+  }
+}
 
 static void
-send_value(C_word x)
+send_scheme_value(C_word x)
 {
   if((x & C_FIXNUM_BIT) != 0)
     C_snprintf(rw_buffer, sizeof(rw_buffer), " %ld", (long)C_unfix(x));
@@ -272,7 +287,7 @@ send_value(C_word x)
 
 
 static void
-send_event(int event, C_char *loc, C_char *val, C_char *cloc, int cln)
+send_event(int event, C_char *loc, C_char *val, C_char *cloc)
 {
   int n;
   int reply, mask;
@@ -284,11 +299,18 @@ send_event(int event, C_char *loc, C_char *val, C_char *cloc, int cln)
   void **stats;
 
   for(;;) {
-    n = C_snprintf(rw_buffer, sizeof(rw_buffer), "(%d \"%s\" \"%s\" \"%s:%d\")\n",
-            event, loc, val, cloc, cln);
+    C_snprintf(rw_buffer, sizeof(rw_buffer), "(%d", event);
     send_string(rw_buffer);
+    send_string_value(loc);
+    send_string_value(val);
+    send_string_value(cloc);
+    send_string(")\n");
 
-    if(socket_read() < 0) terminate("read failed");
+    n = socket_read();
+
+    if(n < 0) terminate("read failed");
+
+    if(n == 0) terminate("debugger disconnected");
 
     /* fprintf(stderr, "<READ: %s>\n", rw_buffer); */
     n = sscanf(rw_buffer, "(%d ", &reply);
@@ -332,11 +354,13 @@ send_event(int event, C_char *loc, C_char *val, C_char *cloc, int cln)
       str = C_strdup(str);
 
       for(dip = unseen_dbg_info_list; dip != NULL; dip = dip->next) {
-          for(dinfo = dip->info; dinfo->loc != NULL; ++dinfo) {
+          for(dinfo = dip->info; dinfo->event; ++dinfo) {
               if(*str == '\0' || strstr(dinfo->val, str)) {
-                  C_snprintf(rw_buffer, sizeof(rw_buffer), "(* %d %d \"%s\" \"%s\")\n",
-                      dbg_info_count++, dinfo->event, dinfo->loc, dinfo->val);
+                  C_snprintf(rw_buffer, sizeof(rw_buffer), "(* %d %d", dbg_info_count++, dinfo->event);
                   send_string(rw_buffer);
+                  send_string_value(dinfo->loc);
+                  send_string_value(dinfo->val);
+                  send_string(")\n");
               }
 
               ++n;
@@ -369,7 +393,7 @@ send_event(int event, C_char *loc, C_char *val, C_char *cloc, int cln)
       send_string("(*");
 
       for(n = 0; n < current_c; ++n)
-        send_value(current_av[ n ]);
+        send_scheme_value(current_av[ n ]);
 
       send_string(")\n");
       break;
@@ -406,7 +430,7 @@ send_event(int event, C_char *loc, C_char *val, C_char *cloc, int cln)
       send_string(rw_buffer);
 
       for(mask = C_header_size(x); n < mask; ++n)
-        send_value(C_block_item(x, n));
+        send_scheme_value(C_block_item(x, n));
 
       send_string(")\n");
       break;
@@ -422,7 +446,7 @@ send_event(int event, C_char *loc, C_char *val, C_char *cloc, int cln)
         send_string("(* UNKNOWN)\n");
       else {
         send_string("(*");
-        send_value(C_symbol_value(x));
+        send_scheme_value(C_symbol_value(x));
         send_string(")\n");
       }
 
@@ -443,23 +467,17 @@ send_event(int event, C_char *loc, C_char *val, C_char *cloc, int cln)
       break;
 
     case C_DEBUG_REPLY_GET_TRACE:
-      str = C_dump_trace(0);
-      C_strlcpy(rw_buffer, "(* \"", sizeof(rw_buffer));
-      ptr = rw_buffer + 4;
+      str = ptr = C_dump_trace(0);
 
-      while(*str != '\0') {
-        if(*str == '\n') {
-          C_strlcpy(ptr, "\")\n", 4);
-          send_string(rw_buffer);
-          C_strlcpy(rw_buffer, "(* \"", sizeof(rw_buffer));
-          ptr = rw_buffer + 4;
-          ++str;
-        }
-        else *(ptr++) = *(str++);
+      while((n = C_strcspn(ptr, "\n"))) {
+        ptr[ n++ ] = '\0';
+        send_string("(* \"");
+        send_string(ptr);
+        send_string("\")\n");
+        ptr += n;
       }
 
-      C_strlcpy(ptr, "\")\n", 4);
-      send_string(rw_buffer);
+      free(str);
       break;
 
     default: terminate("invalid reply code");
@@ -538,7 +556,7 @@ connect_to_debugger()
     return C_SCHEME_FALSE;                     /* failed to connect */
 
   C_snprintf(info, sizeof(info), "%s:%d:%d", C_main_argv[ 0 ], getpid(), C_DEBUG_PROTOCOL_VERSION);
-  send_event(C_DEBUG_CONNECT, info, "", "", 0);
+  send_event(C_DEBUG_CONNECT, info, NULL, NULL);
 #ifndef _WIN32
   C_signal(SIGUSR2, interrupt_signal_handler);
 #endif
@@ -547,21 +565,18 @@ connect_to_debugger()
 
 
 static C_word
-debug_event_hook(C_DEBUG_INFO *cell, C_word c, C_word *av, char *cloc, int cln)
+debug_event_hook(C_DEBUG_INFO *cell, C_word c, C_word *av, C_char *cloc)
 {
   if(socket_fd != 0) {
     if(cell->enabled || interrupted || ((1 << cell->event) & event_mask) != 0 ) {
-      /* fprintf(stderr, "event: %s:%d\n", cloc, cln); */
+      /* fprintf(stderr, "event: %s\n", cloc); */
       current_c = c;
       current_av = av;
-      send_event(interrupted ? C_DEBUG_INTERRUPTED : cell->event, cell->loc,
-        cell->val, cloc, cln);
+      send_event(interrupted ? C_DEBUG_INTERRUPTED : cell->event, cell->loc, cell->val, cloc);
       interrupted = 0;
     }
   }
 
-  if(cell->event == C_DEBUG_CALL) C_trace(cell->val);
-
   return C_SCHEME_UNDEFINED;
 }
 
