@@ -1,8 +1,9 @@
 ;;;; compiler-tests.scm
 
 
-(import foreign)
-(use srfi-4)
+(import (chicken bitwise) (chicken flonum) (chicken foreign)
+	(chicken condition) (srfi 4))
+(import-for-syntax (chicken syntax) (chicken string))
 
 ;; test dropping of previous toplevel assignments
 
@@ -13,11 +14,15 @@
 
 (assert (eq? 'ok (foo)))
 
+(assert (= 1 (foreign-type-size "char")))
+(let* ((words->bytes (foreign-lambda int "C_wordstobytes" int))
+       (bytes-in-a-word (words->bytes 1)))
+  (assert (= bytes-in-a-word (foreign-type-size "C_word"))))
 
 ;; test hiding of unexported toplevel variables
 
 (module foo (bar)
-  (import scheme chicken)
+  (import scheme chicken.base)
   (declare (hide bar))
   (define (bar x) (+ x 1)))
 
@@ -48,7 +53,7 @@
 (module
  x
  (bar)
- (import scheme chicken foreign)
+ (import scheme chicken.base chicken.foreign)
 
  (define (bar n)
   (let-location ((off integer 0))
@@ -64,6 +69,19 @@
 (import x)
 (assert (= 1 ((bar 42))))
 
+
+;; Test custom foreign type conversions
+
+(module y (my-add1)
+  (import scheme (chicken base) (chicken foreign))
+
+  (define-foreign-type my-int integer add1 sub1)
+
+  (define my-add1 (foreign-lambda* my-int ((my-int x)) "C_return(x+1);")))
+
+(import y)
+(assert (= 2 (my-add1 1)))
+
 ;;; rev. 14574 (reported by Peter Bex)
 ;
 ; - type specifiers in foreign-lambda in macros are incorrectly renamed
@@ -102,10 +120,12 @@
 ;; Type specifiers and variable names in foreign-lambda in macros
 ;; are incorrectly renamed in modules, too.
 (foreign-declare "void foo(void *abc) { printf(\"hi\\n\"); }")
+;; This is silly but at least it ensures we can represent enum values
+(foreign-declare "enum intlimits {min=INT_MIN, zero=0, max=INT_MAX};")
 
 (module foo ()
-  (import chicken scheme foreign) ; "chicken" includes an export for "void"
-  
+  (import scheme chicken.base chicken.foreign)
+
   (let-syntax ((fl
                 (syntax-rules ()
                   ((_)
@@ -141,14 +161,14 @@
 ;; Unused arguments in foreign callback wrappers are not optimized away (#584)
 (module bla (foo)
 
-(import chicken scheme foreign)
+(import (prefix scheme s:) (only chicken.base assert) chicken.foreign)
 
 (define-external
   (blabla (int a) (c-string b) (int c) (int d) (c-string e) (int f))
   int
   f)
 
-(define (foo) ((foreign-safe-lambda* int () "C_return(blabla(1, \"2\", 3, 4, \"5\", 6));")))
+(s:define (foo) ((foreign-safe-lambda* int () "C_return(blabla(1, \"2\", 3, 4, \"5\", 6));")))
 
 (assert (location blabla))
 )
@@ -237,6 +257,16 @@
                       (set! outer-bar inner-bar) 
                       (outer-bar '#f))))) 
 
+;; Found by Claude Marinier: Huge literals with a length which need
+;; more than 3 bytes to encode would get silently truncated.  We'll
+;; prevent constant-folding if it would lead to such large literals.
+(let* ((bignum (expt 2 70000000))
+       ;; This prevents complete evaluation at compile-time
+       (unknown-bignum ((foreign-lambda* scheme-object
+			    ((scheme-object n)) "C_return(n);") bignum)))
+  (assert (equal? 70000001 (integer-length unknown-bignum))))
+
+
 ;; Test that encode-literal/decode-literal use the proper functions
 ;; to decode number literals.
 (assert (equal? '(+inf.0 -inf.0) (list (fp/ 1.0 0.0) (fp/ -1.0 0.0))))
@@ -255,20 +285,107 @@
            (string->number "179769313486231570814527423731704356798070567525844996598917476803157260780028538760589558632766878171540458953514382464234321326889464182768467546703537516986049910576551282076245490090389328944075868508455133942304583236903222948165808559332123348274797826204144723168738177180919299881250404026184124858368.0")))
 
 ;; #955: unsigned-integer64 arg returned magnitude instead of Scheme object.
-#+64bit
-(assert (= #xAB54A98CEB1F0AD2
-           ((foreign-lambda* unsigned-integer64 ((unsigned-integer64 x))
-              "C_return(x);")
-            #xAB54A98CEB1F0AD2)))
+(assert (eqv? #xAB54A98CEB1F0AD2
+	      ((foreign-lambda* unsigned-integer64 ((unsigned-integer64 x))
+		 "C_return(x);")
+	       #xAB54A98CEB1F0AD2)))
+
+
+;; Test the maximum and minimum values of the FFI's integer types
+(define-syntax test-ffi-type-limits
+  (syntax-rules (signed unsigned)
+    ((_ ?type-name unsigned ?bits)
+     (let ((limit (arithmetic-shift 1 ?bits)))
+       (print "Testing unsigned FFI type \"" '?type-name "\" (" ?bits " bits):")
+       (print "Can hold maximum value " (sub1 limit) "...")
+       (assert
+	(eqv? (sub1 limit)
+	      ((foreign-lambda* ?type-name ((?type-name x))
+		 "C_return(x);") (sub1 limit))))
+       (print "Cannot hold one more than maximum value, " limit "...")
+       (assert
+	(handle-exceptions exn #t
+	  (begin ((foreign-lambda* ?type-name ((?type-name x))
+		    "C_return(x);") limit)
+		 #f)))
+       (print "Cannot hold -1 (any fixnum negative value)")
+       (assert
+	(handle-exceptions exn #t
+	  (begin ((foreign-lambda* ?type-name ((?type-name x))
+		    "C_return(x);") -1)
+		 #f)))
+       (print "Cannot hold -2^64 (any bignum negative value < smallest int64)")
+       (assert
+	(handle-exceptions exn #t
+	  (begin ((foreign-lambda* ?type-name ((?type-name x))
+		    "C_return(x);") #x-10000000000000000)
+		 #f)))))
+    ((_ ?type-name signed ?bits)
+     (let ((limit (arithmetic-shift 1 (sub1 ?bits))))
+       (print "Testing signed FFI type \"" '?type-name "\" (" ?bits " bits):")
+       (print "Can hold maximum value " (sub1 limit) "...")
+       (assert
+	(eqv? (sub1 limit)
+	      ((foreign-lambda* ?type-name ((?type-name x))
+		 "C_return(x);") (sub1 limit))))
+       (print "Can hold minimum value " (- limit) "...")
+       (assert
+	(eqv? (- limit)
+	      ((foreign-lambda* ?type-name ((?type-name x))
+		 "C_return(x);") (- limit))))
+       (print "Cannot hold one more than maximum value " limit "...")
+       (assert
+	(handle-exceptions exn #t
+	  (begin ((foreign-lambda* ?type-name ((?type-name x))
+		    "C_return(x);") limit)
+		 #f)))
+       (print "Cannot hold one less than minimum value " (- limit) "...")
+       (assert
+	(handle-exceptions exn #t
+	  (begin ((foreign-lambda* ?type-name ((?type-name x))
+		    "C_return(x);") (sub1 (- limit)))
+		 #f)))))))
+
+(test-ffi-type-limits unsigned-integer32 unsigned 32)
+(test-ffi-type-limits integer32 signed 32)
+
+(test-ffi-type-limits unsigned-integer64 unsigned 64)
+(test-ffi-type-limits integer64 signed 64)
+
+(test-ffi-type-limits
+ unsigned-integer unsigned
+ (foreign-value "sizeof(unsigned int) * CHAR_BIT" int))
+
+(test-ffi-type-limits
+ integer signed (foreign-value "sizeof(int) * CHAR_BIT" int))
+
+(test-ffi-type-limits
+ (enum intlimits) signed
+ (foreign-value "sizeof(enum intlimits) * CHAR_BIT" int))
+
+(test-ffi-type-limits
+ unsigned-long unsigned
+ (foreign-value "sizeof(unsigned long) * CHAR_BIT" int))
+
+(test-ffi-type-limits
+ long signed (foreign-value "sizeof(long) * CHAR_BIT" int))
+
+(test-ffi-type-limits
+ ssize_t signed (foreign-value "sizeof(ssize_t) * CHAR_BIT" int))
+
+(test-ffi-type-limits
+ size_t unsigned (foreign-value "sizeof(size_t) * CHAR_BIT" int))
+
 
 ;; #1059: foreign vector types use wrong lolevel accessors, causing
 ;; paranoid DEBUGBUILD assertions to fail.
 (define-syntax srfi-4-vector-length
-  (lambda (e r c)
-    (let* ((type (symbol->string (strip-syntax (cadr e))))
-           (base-type (string-translate* type '(("nonnull-" . ""))))
-           (length-procedure-name (string-append base-type "-length")))
-     `(,(string->symbol length-procedure-name) ,(caddr e)))))
+  (er-macro-transformer
+   (lambda (e r c)
+     (let* ((type (symbol->string (strip-syntax (cadr e))))
+	    (base-type (string-translate* type '(("nonnull-" . ""))))
+	    (length-procedure-name (string-append base-type "-length")))
+       `(,(string->symbol length-procedure-name) ,(caddr e))))))
 
 (define-syntax s4v-sum
   (syntax-rules ()
@@ -289,15 +406,19 @@
 (assert (= 10 (s4v-sum "integer" u8vector '#u8(1 2 3 4))))
 (assert (= 10 (s4v-sum "integer" u16vector '#u16(1 2 3 4))))
 (assert (= 10 (s4v-sum "integer" u32vector '#u32(1 2 3 4))))
+(assert (= 10 (s4v-sum "integer" s64vector '#s64(1 2 3 4))))
 (assert (= 10 (s4v-sum "integer" nonnull-u8vector '#u8(1 2 3 4))))
 (assert (= 10 (s4v-sum "integer" nonnull-u16vector '#u16(1 2 3 4))))
 (assert (= 10 (s4v-sum "integer" nonnull-u32vector '#u32(1 2 3 4))))
+(assert (= 10 (s4v-sum "integer" nonnull-u64vector '#u64(1 2 3 4))))
 (assert (= -10 (s4v-sum "integer" s8vector '#s8(-1 -2 -3 -4))))
 (assert (= -10 (s4v-sum "integer" s16vector '#s16(-1 -2 -3 -4))))
 (assert (= -10 (s4v-sum "integer" s32vector '#s32(-1 -2 -3 -4))))
+(assert (= -10 (s4v-sum "integer" s64vector '#s64(-1 -2 -3 -4))))
 (assert (= -10 (s4v-sum "integer" nonnull-s8vector '#s8(-1 -2 -3 -4))))
 (assert (= -10 (s4v-sum "integer" nonnull-s16vector '#s16(-1 -2 -3 -4))))
 (assert (= -10 (s4v-sum "integer" nonnull-s32vector '#s32(-1 -2 -3 -4))))
+(assert (= -10 (s4v-sum "integer" nonnull-s64vector '#s64(-1 -2 -3 -4))))
 (assert (= 12.0 (s4v-sum "float" f32vector '#f32(1.5 2.5 3.5 4.5))))
 (assert (= 12.0 (s4v-sum "float" f64vector '#f64(1.5 2.5 3.5 4.5))))
 (assert (= 12.0 (s4v-sum "float" nonnull-f32vector '#f32(1.5 2.5 3.5 4.5))))
@@ -309,7 +430,7 @@
 ;; procedures not getting replaced with explicitly consed rest
 ;; list when the procedures themselves were hidden.
 (module explicitly-consed-rest-args-bug (do-it also-do-it)
- (import scheme chicken)
+ (import scheme chicken.base chicken.type)
 
  (: get-value (* * #!rest * --> *))
  (define (get-value x y . rest)
