@@ -1,12 +1,25 @@
 ;;;; library-tests.scm
 
-(use srfi-1 extras ports)
+(import chicken.blob chicken.bitwise chicken.fixnum chicken.flonum
+	chicken.keyword chicken.port chicken.condition)
 
 (define-syntax assert-fail
   (syntax-rules ()
     ((_ exp)
      (assert (handle-exceptions ex #t exp #f)))))
 
+(define (list-tabulate n proc)
+  (let loop ((i 0))
+    (if (fx>= i n)
+	'()
+	(cons (proc i) (loop (fx+ i 1))))))
+
+(define (every pred lst)
+  (let loop ((lst lst))
+    (cond ((null? lst))
+	  ((not (pred (car lst))) #f)
+	  (else (loop (cdr lst))))))
+
 ;; numbers
 
 (assert (not (not 3)))
@@ -54,12 +67,13 @@
 (assert (inexact? 1.1))
 (assert-fail (inexact? 'foo))
 
+;; Division by inexact zero used to fail, but now it returns +inf.0
 (assert-fail (/ 1 1 0))
-(assert-fail (/ 1 1 0.0))
-(assert-fail (/ 1 0.0))
+(assert (eqv? +inf.0 (/ 1 1 0.0)))
+(assert (eqv? +inf.0 (/ 1 0.0)))
 (assert-fail (/ 1 0))
 (assert-fail (/ 0))
-(assert-fail (/ 0.0))
+(assert (eqv? +inf.0 (/ 0.0)))
 
 (assert (fixnum? (/ 1)))
 
@@ -90,12 +104,12 @@
 ;;; A few denormalised numbers, cribbed from NetBSD ATF tests for ldexp():
 ;; On some machines/OSes these tests fail due to missing hardware support
 ;; and sometimes due to broken libc/libm support, so we have disabled them.
-(assert (equal? 1.0 (numerator 1.1125369292536006915451e-308)))
-(assert (equal? +inf.0 (denominator 1.1125369292536006915451e-308)))
-(assert (equal? -1.0 (numerator -5.5626846462680034577256e-309)))
-(assert (equal? +inf.0 (denominator -5.5626846462680034577256e-309)))
-(assert (equal? 1.0 (numerator 4.9406564584124654417657e-324)))
-(assert (equal? +inf.0 (denominator 4.9406564584124654417657e-324)))
+;(assert (equal? 1.0 (numerator 1.1125369292536006915451e-308)))
+;(assert (equal? +inf.0 (denominator 1.1125369292536006915451e-308)))
+;(assert (equal? -1.0 (numerator -5.5626846462680034577256e-309)))
+;(assert (equal? +inf.0 (denominator -5.5626846462680034577256e-309)))
+;(assert (equal? 1.0 (numerator 4.9406564584124654417657e-324)))
+;(assert (equal? +inf.0 (denominator 4.9406564584124654417657e-324)))
 
 (assert (equal? 4.0 (denominator -1.25)))
 (assert (equal? 1e10 (numerator 1e10)))
@@ -226,8 +240,8 @@
 ;; by Christian Kellermann
 (assert 
  (equal?
-  (map (lambda (n) (number->string 32 n)) (iota 15 2))
-  '("100000" "1012" "200" "112" "52" "44" "40" "35" "32" "2A" "28" "26" "24" "22" "20")))
+  (map (lambda (n) (number->string 32 n)) (list-tabulate 15 (cut + 2 <>)))
+  '("100000" "1012" "200" "112" "52" "44" "40" "35" "32" "2a" "28" "26" "24" "22" "20")))
 
 
 ;; string->number conversion
@@ -277,7 +291,6 @@
 	      (write (string->symbol "3"))))
 	read)))
 
-
 ;;; escaped symbol syntax
 
 (assert (string=? "abc" (symbol->string '|abc|)))
@@ -307,6 +320,18 @@
   ;; "Unterminated string" (unterminated identifier?)
   (assert-fail (with-input-from-string "a|Bc" read)))
 
+;;; Old style qualified low byte, see #1077
+
+(assert (string=? "##foo#bar" (symbol->string '|##foo#bar|)))
+(assert (string=? "##foo#bar" (symbol->string '##foo#bar)))
+(assert (eq? '##foo#bar '|##foo#bar|))
+
+(assert (string=? "|\\x0a|" (with-output-to-string (lambda () (write '|\n|)))))
+;; #1576
+(assert (string=? "|\\x00foo|" (with-output-to-string (lambda () (write '|\000foo|)))))
+(assert (not (keyword? '|\000foo|)))
+(assert (string=? "|###foo#bar|" (with-output-to-string (lambda () (write '|###foo#bar|)))))
+
 ;;; Paren synonyms
 
 (parameterize ((parentheses-synonyms #f))
@@ -322,25 +347,100 @@
 
 (parameterize ((keyword-style #:suffix))
   (assert (string=? "abc:" (symbol->string (with-input-from-string "|abc:|" read))))
-  (assert (string=? "abc" (symbol->string (with-input-from-string "|abc|:" read)))) ; keyword
-  (let ((kw (with-input-from-string "|foo bar|:" read)))
+  (assert (string=? "abc" (keyword->string (with-input-from-string "|abc|:" read)))) ; keyword
+  (let ((kw (with-input-from-string "|foo bar|:" read))
+	(sym1 (with-input-from-string "|foo:|" read))
+	(sym2 (with-input-from-string "|:foo|" read)))
+
+    (assert (symbol? sym1))
+    (assert (not (keyword? sym1)))
+
+    (assert (symbol? sym2))
+    (assert (not (keyword? sym2)))
+
+    (assert (keyword? kw))
+    (assert (not (symbol? kw)))
+
     (assert (eq? kw (with-input-from-string "#:|foo bar|" read)))
-    (assert (string=? "foo bar" (symbol->string kw)))
+    (assert (string=? "foo bar" (keyword->string kw)))
+    (assert (string=? "foo:" (symbol->string sym1)))
+    (assert (string=? ":foo" (symbol->string sym2)))
+
     (assert (string=? "foo bar:"
 		      (with-output-to-string (lambda () (display kw)))))
-    (assert (string=? "|foo bar|:"
-		      (with-output-to-string (lambda () (write kw)))))))
+    (assert (string=? "#:|foo bar|"
+		      (with-output-to-string (lambda () (write kw)))))
+
+    (assert (string=? "|foo:|"
+		      (with-output-to-string (lambda () (write sym1)))))
+    ;; Regardless of keyword style, symbols must be quoted to avoid
+    ;; issues when reading it back with a different keyword style.
+    (assert (string=? "|:foo|"
+		      (with-output-to-string (lambda () (write sym2)))))))
 
 (parameterize ((keyword-style #:prefix))
-  (assert (string=? "abc" (symbol->string (with-input-from-string ":|abc|" read))))
+  (assert (string=? "abc" (keyword->string (with-input-from-string ":|abc|" read))))
   (assert (string=? ":abc" (symbol->string (with-input-from-string "|:abc|" read))))
-  (let ((kw (with-input-from-string ":|foo bar|" read)))
+  (let ((kw (with-input-from-string ":|foo bar|" read))
+	(sym1 (with-input-from-string "|:foo|" read))
+	(sym2 (with-input-from-string "|foo:|" read)))
+
+    (assert (symbol? sym1))
+    (assert (not (keyword? sym1)))
+
+    (assert (symbol? sym2))
+    (assert (not (keyword? sym2)))
+
+    (assert (keyword? kw))
+    (assert (not (symbol? kw)))
+
     (assert (eq? kw (with-input-from-string "#:|foo bar|" read)))
-    (assert (string=? "foo bar" (symbol->string kw)))
+    (assert (string=? "foo bar" (keyword->string kw)))
+    (assert (string=? ":foo" (symbol->string sym1)))
+    (assert (string=? "foo:" (symbol->string sym2)))
+
     (assert (string=? ":foo bar"
 		      (with-output-to-string (lambda () (display kw)))))
-    (assert (string=? ":|foo bar|"
-		      (with-output-to-string (lambda () (write kw)))))))
+    (assert (string=? "#:|foo bar|"
+		      (with-output-to-string (lambda () (write kw)))))
+
+    (assert (string=? "|:foo|"
+		      (with-output-to-string (lambda () (write sym1)))))
+    ;; Regardless of keyword style, symbols must be quoted to avoid
+    ;; issues when reading it back with a different keyword style.
+    (assert (string=? "|foo:|"
+		      (with-output-to-string (lambda () (write sym2)))))))
+
+(parameterize ((keyword-style #:none))
+  (let ((kw (with-input-from-string "#:|foo bar|" read))
+	(sym1 (with-input-from-string "|:foo|" read))
+	(sym2 (with-input-from-string "|foo:|" read)))
+
+    (assert (symbol? sym1))
+    (assert (not (keyword? sym1)))
+
+    (assert (symbol? sym2))
+    (assert (not (keyword? sym2)))
+
+    (assert (keyword? kw))
+    (assert (not (symbol? kw)))
+
+    (assert (eq? kw (string->keyword "foo bar"))
+    (assert (string=? "foo bar" (keyword->string kw)))
+    (assert (string=? ":foo" (symbol->string sym1)))
+    (assert (string=? "foo:" (symbol->string sym2)))
+
+    (assert (string=? ":foo"
+		      (with-output-to-string (lambda () (display kw)))))
+    (assert (string=? "#:|foo bar|"
+		      (with-output-to-string (lambda () (write kw)))))
+
+    ;; Regardless of keyword style, symbols must be quoted to avoid
+    ;; issues when reading it back with a different keyword style.
+    (assert (string=? "|:foo|"
+		      (with-output-to-string (lambda () (write sym1)))))
+    (assert (string=? "|foo:|"
+		      (with-output-to-string (lambda () (write sym2))))))))
 
 (assert (eq? '|#:| (string->symbol "#:")))
 (assert-fail (with-input-from-string "#:" read)) ; empty keyword
@@ -387,15 +487,26 @@
 (assert-fail (with-input-from-string "#:" read))
 
 (let ((empty-kw (with-input-from-string "#:||" read)))
+  (assert (not (symbol? empty-kw)))
   (assert (keyword? empty-kw))
   (assert (string=? "" (keyword->string empty-kw))))
 
+;; TODO: It should eventually be possible to distinguish these (#1077)
+#;(let ((nul-sym (with-input-from-string "|\\x00|" read)))
+  (assert (not (keyword? nul-sym)))
+  (assert (string=? "\x00" (symbol->string nul-sym))))
+
 (assert (keyword? (with-input-from-string "42:" read)))
 (assert (keyword? (with-input-from-string ".:" read)))
 
 (assert (equal? (cons 1 2) (with-input-from-string "(1 . 2)" read)))
 (assert (every keyword? (with-input-from-string "(42: abc: .: #:: ::)" read)))
 
+;; symbols and keywords are now distinct
+(assert (not (symbol? #:foo)))
+(assert (not (symbol? (string->keyword "foo"))))
+(assert (not (keyword? 'foo)))
+(assert (not (keyword? (string->symbol "foo"))))
 
 ;;; reading unterminated objects
 
@@ -479,6 +590,12 @@ A
 (assert (string-ci<? "foo\x00a" "foo\x00B"))
 (assert (string-ci>? "foo\x00b" "foo\x00A"))
 
+;; reported by Nils Holm (#1534)
+;; https://groups.google.com/group/comp.lang.scheme/t/6b8be06b84b39a7
+(assert (not (string-ci<=? "test" "tes")))
+(assert (string-ci>=? "test" "tes"))
+
+
 ;;; getter-with-setter
 
 (define foo
