@@ -4,16 +4,17 @@
 # - Note: this needs a proper shell, so it will not work with plain mingw
 #   (just the compiler and the Windows shell, without MSYS)
 
-
 set -e
 if test -z "$MSYSTEM"; then
     TEST_DIR=`pwd`
+    PATH_SEP=':'
 else
     # Use Windows-native format with drive letters instead of awkward
     # MSYS /c/blabla "pseudo-paths" which break when used in syscalls.
     TEST_DIR=`pwd -W`
+    PATH_SEP=';'
 fi
-OS_NAME=`uname -s`
+
 DYLD_LIBRARY_PATH=${TEST_DIR}/..
 LD_LIBRARY_PATH=${TEST_DIR}/..
 LIBRARY_PATH=${TEST_DIR}/..:${LIBRARY_PATH}
@@ -21,6 +22,7 @@ LIBRARY_PATH=${TEST_DIR}/..:${LIBRARY_PATH}
 # the binary are read by the OS itself, which uses $PATH (mingw too)
 # Oddly, prefixing .. with ${TEST_DIR}/ does _not_ work on mingw!
 PATH=..:${PATH}
+
 export DYLD_LIBRARY_PATH LD_LIBRARY_PATH LIBRARY_PATH PATH
 
 case `uname` in
@@ -30,92 +32,102 @@ case `uname` in
 		DIFF_OPTS=-bu ;;
 esac
 
-rm -fr test-repository
-mkdir -p test-repository
+CHICKEN=${TEST_DIR}/../${PROGRAM_PREFIX}chicken${PROGRAM_SUFFIX}
+CHICKEN_PROFILE=${TEST_DIR}/../${PROGRAM_PREFIX}chicken-profile${PROGRAM_SUFFIX}
+CHICKEN_INSTALL=${TEST_DIR}/../${PROGRAM_PREFIX}chicken-install${PROGRAM_SUFFIX}
+CHICKEN_UNINSTALL=${TEST_DIR}/../${PROGRAM_PREFIX}chicken-uninstall${PROGRAM_SUFFIX}
+CHICKEN_INSTALL_REPOSITORY=${TEST_DIR}/test-repository
+CHICKEN_REPOSITORY_PATH="${TEST_DIR}/..${PATH_SEP}${CHICKEN_INSTALL_REPOSITORY}"
 
-# copy files into test-repository (by hand to avoid calling `chicken-install'):
-
-for x in setup-api.so setup-api.import.so setup-download.so \
-      setup-download.import.so chicken.import.so lolevel.import.so \
-      srfi-1.import.so srfi-4.import.so data-structures.import.so \
-      ports.import.so files.import.so posix.import.so \
-      srfi-13.import.so srfi-69.import.so extras.import.so \
-      irregex.import.so srfi-14.import.so tcp.import.so \
-      foreign.import.so srfi-18.import.so \
-      utils.import.so csi.import.so irregex.import.so types.db; do
-  cp ../$x test-repository
-done
-
-CHICKEN_REPOSITORY=${TEST_DIR}/test-repository
-CHICKEN=${TEST_DIR}/../chicken
-CHICKEN_INSTALL=${TEST_DIR}/../chicken-install
-CHICKEN_UNINSTALL=${TEST_DIR}/../chicken-uninstall
-ASMFLAGS=
-FAST_OPTIONS="-O5 -d0 -b -disable-interrupts"
-COMPILE_OPTIONS="-compiler ${TEST_DIR}/../chicken -v -I${TEST_DIR}/.. -L${TEST_DIR}/.. -rpath ${TEST_DIR}/.. -include-path ${TEST_DIR}/.."
-
-TEST_DIR_SEXPR=`../csi -n -include-path .. -e "(use posix) (write (current-directory))"`
-SETUP_PREFIX="-e (use files setup-api)"
-SETUP_PREFIX="${SETUP_PREFIX} -e (register-program \"csc\" (make-pathname ${TEST_DIR_SEXPR} \"../csc\"))"
-SETUP_PREFIX="${SETUP_PREFIX} -e (register-program \"chicken\" (make-pathname ${TEST_DIR_SEXPR} \"../chicken\"))"
-SETUP_PREFIX="${SETUP_PREFIX} -e (register-program \"csi\" (make-pathname ${TEST_DIR_SEXPR} \"../csi\"))"
+export CHICKEN_INSTALL_REPOSITORY CHICKEN_REPOSITORY_PATH
 
 TYPESDB=../types.db
-cp $TYPESDB test-repository/types.db
+COMPILE_OPTIONS="-v -compiler ${CHICKEN} -I${TEST_DIR}/.. -L${TEST_DIR}/.. -include-path ${TEST_DIR}/.. -libdir ${TEST_DIR}/.. -rpath ${TEST_DIR}/.."
 
+compile="../${PROGRAM_PREFIX}csc${PROGRAM_SUFFIX} ${COMPILE_OPTIONS} -o a.out -types ${TYPESDB} -ignore-repository"
+compile_r="../${PROGRAM_PREFIX}csc${PROGRAM_SUFFIX} ${COMPILE_OPTIONS}"
+compile_s="../${PROGRAM_PREFIX}csc${PROGRAM_SUFFIX} ${COMPILE_OPTIONS} -s -types ${TYPESDB} -ignore-repository"
+interpret="../${PROGRAM_PREFIX}csi${PROGRAM_SUFFIX} -n -include-path ${TEST_DIR}/.."
 time=time
-compile="../csc -types ${TYPESDB} -ignore-repository ${COMPILE_OPTIONS} -o a.out"
-compile2="../csc -compiler $CHICKEN -v -I${TEST_DIR}/.. -L${TEST_DIR}/.. -include-path ${TEST_DIR}/.."
-compile_s="../csc -s -types ${TYPESDB} -ignore-repository ${COMPILE_OPTIONS} -v -I${TEST_DIR}/.. -L${TEST_DIR}/.. -include-path ${TEST_DIR}/.."
-interpret="../csi -n -include-path ${TEST_DIR}/.."
 
-# Check for a `time' command, since some systems don't ship with a
-# time(1) or shell builtin and we also can't portably rely on which(1),
-# etc. NOTE `time' must be called from a variable here.
+# Check for a "time" command, since some systems don't ship with a
+# time(1) or shell builtin and we also can't portably rely on `which',
+# `command', etc. NOTE "time" must be called from a variable here.
 set +e
 $time true >/dev/null 2>/dev/null
-if [ $? -eq 127 ]; then time=; fi
+test $? -eq 127 && time=
 set -e
 
-rm -f *.exe *.so *.o *.import.* a.out ../foo.import.*
+rm -fr *.exe *.so *.o *.obj *.out *.import.* ../foo.import.* test-repository
+mkdir -p test-repository
+cp $TYPESDB test-repository/types.db
+
+echo "======================================== repository search path ..."
+if test -z "$MSYSTEM"; then
+  export -p >./env.out
+  unset CHICKEN_REPOSITORY_PATH
+  $interpret -s repository-path-default.scm
+  . ./env.out
+else
+  env -u CHICKEN_REPOSITORY_PATH $interpret -s repository-path-default.scm
+fi
+$compile_s sample-module.scm -j sample-module
+cp sample-module.so $CHICKEN_INSTALL_REPOSITORY
+cp sample-module.import.scm $CHICKEN_INSTALL_REPOSITORY
+$interpret -s repository-path.scm "${TEST_DIR}/.." "${TEST_DIR}/test-repository"
+
+echo "======================================== types.db consistency ..."
+$interpret -s types-db-consistency.scm ${TYPESDB}
 
 echo "======================================== version tests ..."
-$interpret -s version-tests.scm
+$compile version-tests.scm
+./a.out
 
 echo "======================================== compiler tests ..."
 $compile compiler-tests.scm
 ./a.out
 
+echo "======================================== csc tests ..."
+$interpret -s csc-tests.scm
+
 echo "======================================== compiler inlining tests  ..."
 $compile inlining-tests.scm -optimize-level 3
 ./a.out
 
+echo "======================================== compiler message tests ..."
+$compile -analyze-only messages-test.scm 2>messages.out
+diff $DIFF_OPTS messages.expected messages.out
+
+echo "======================================== optimizer tests  ..."
+$compile clustering-tests.scm -clustering
+./a.out
+
+echo "======================================== profiler tests ..."
+$compile null.scm -profile -profile-name TEST.profile
+./a.out
+$CHICKEN_PROFILE TEST.profile
+
 echo "======================================== scrutiny tests ..."
+$compile scrutinizer-tests.scm -analyze-only
 $compile typematch-tests.scm -specialize -no-warnings
 ./a.out
 
-$compile scrutiny-tests.scm -analyze-only -verbose 2>scrutiny.out
-$compile specialization-tests.scm -analyze-only -verbose -specialize 2>specialization.out
+$compile test-scrutinizer-message-format.scm -A -specialize 2>scrutinizer-message-format.out || true
+$compile scrutiny-tests.scm -A 2>scrutiny.out
+$compile scrutiny-tests-2.scm -A 2>scrutiny-2.out
+$compile specialization-tests.scm -A -specialize 2>specialization.out
 
-# these are sensitive to gensym-names, so make them optional
-if test \! -f scrutiny.expected; then
-    cp scrutiny.out scrutiny.expected
-fi
-if test \! -f specialization.expected; then
-    cp specialization.out specialization.expected
-fi
+# Replace foo123 -> fooXX so gensyms don't trigger failures
+$compile redact-gensyms.scm -o redact-gensyms
+./redact-gensyms "tmp,g,scm:,a,b" < scrutinizer-message-format.out > scrutinizer-message-format.redacted
+./redact-gensyms < scrutiny-2.out > scrutiny-2.redacted
+./redact-gensyms < scrutiny.out > scrutiny.redacted
+./redact-gensyms < specialization.out > specialization.redacted
 
-diff $DIFF_OPTS scrutiny.expected scrutiny.out
-diff $DIFF_OPTS specialization.expected specialization.out
-
-$compile scrutiny-tests-2.scm -A -scrutinize -analyze-only 2>scrutiny-2.out -verbose
-
-# this is sensitive to gensym-names, so make it optional
-if test \! -f scrutiny-2.expected; then
-    cp scrutiny-2.out scrutiny-2.expected
-fi
-
-diff $DIFF_OPTS scrutiny-2.expected scrutiny-2.out
+diff $DIFF_OPTS scrutinizer-message-format.expected scrutinizer-message-format.redacted
+diff $DIFF_OPTS scrutiny.expected scrutiny.redacted
+diff $DIFF_OPTS scrutiny-2.expected scrutiny-2.redacted
+diff $DIFF_OPTS specialization.expected specialization.redacted
 
 $compile scrutiny-tests-3.scm -specialize -block
 ./a.out
@@ -125,23 +137,31 @@ $compile scrutiny-tests-strict.scm -strict-types -specialize
 
 echo "======================================== specialization tests ..."
 rm -f foo.types foo.import.*
-$compile specialization-test-1.scm -emit-type-file foo.types -specialize \
+$compile specialization-test-1.scm -emit-types-file foo.types -specialize \
   -debug ox -emit-import-library foo
 ./a.out
-$compile specialization-test-2.scm -types foo.types -types specialization-test-2.types -feature chicken-bootstrap -specialize -debug ox
+$compile specialization-test-2.scm -types foo.types -types specialization-test-2.types -specialize -debug ox
 ./a.out
 rm -f foo.types foo.import.*
 
 echo "======================================== specialization benchmark ..."
-$compile fft.scm -O2 -local -d0 -disable-interrupts -b -o fft1
-$compile fft.scm -O2 -local -specialize -debug x -d0 -disable-interrupts -b -o fft2 -specialize
+$compile fft.scm -O2 -local -d0 -disable-interrupts -b -o fft1.out
+$compile fft.scm -O2 -local -specialize -debug x -d0 -disable-interrupts -b -o fft2.out -specialize
 echo "normal:"
-$time ./fft1 1000 7
+$time ./fft1.out 1000 7
 echo "specialized:"
-$time ./fft2 1000 7
+$time ./fft2.out 1000 7
+
+echo "======================================== unboxing benchmark ..."
+$compile fft.scm -O2 -local -d0 -disable-interrupts -b -o fft1.out
+$compile fft.scm -O4 -debug x -d0 -disable-interrupts -b -o fft2.out
+echo "normal:"
+$time ./fft1.out 1000 7
+echo "unboxing:"
+$time ./fft2.out 1000 7
 
 echo "======================================== callback tests ..."
-$compile callback-tests.scm
+$compile -extend c-id-valid.scm callback-tests.scm
 ./a.out
 
 if ./a.out twice; then
@@ -182,8 +202,6 @@ diff $DIFF_OPTS dwindtst.expected dwindtst.out
 $compile dwindtst.scm
 ./a.out >dwindtst.out
 diff $DIFF_OPTS dwindtst.expected dwindtst.out
-echo "*** Skipping \"feeley-dynwind\" for now ***"
-# $interpret -s feeley-dynwind.scm
 
 echo "======================================== lolevel tests ..."
 $interpret -s lolevel-tests.scm
@@ -211,10 +229,10 @@ $compile syntax-tests-2.scm
 ./a.out
 
 echo "======================================== meta-syntax tests ..."
-$interpret -bnq meta-syntax-test.scm -e '(import foo)' -e "(assert (equal? '((1)) (bar 1 2)))" -e "(assert (equal? '(list 1 2 3) (listify)))" -e "(import foo-usage)" -e "(assert (equal? '(1) (foo-user)))"
+$interpret -bnq meta-syntax-test.scm -e '(import foo)' -e "(assert (equal? '((1)) (bar 1 2)))" -e "(assert (equal? '(list 1 2 3) (listify)))" -e "(import test-import-syntax-for-syntax)" -e "(assert (equal? '(1) (test)))" -e "(import test-begin-for-syntax)" -e "(assert (equal? '(1) (test)))"
 $compile_s meta-syntax-test.scm -j foo
 $compile_s foo.import.scm
-$interpret -bnq -e '(require-library meta-syntax-test)' -e '(import foo)' -e "(assert (equal? '((1)) (bar 1 2)))" -e "(assert (equal? '(list 1 2 3) (listify)))" -e "(import foo-usage)" -e "(assert (equal? '(1) (foo-user)))"
+$interpret -bnq meta-syntax-test.scm -e '(import foo)' -e "(assert (equal? '((1)) (bar 1 2)))" -e "(assert (equal? '(list 1 2 3) (listify)))" -e "(import test-import-syntax-for-syntax)" -e "(assert (equal? '(1) (test)))" -e "(import test-begin-for-syntax)" -e "(assert (equal? '(1) (test)))"
 
 echo "======================================== reexport tests ..."
 $interpret -bnq reexport-tests.scm
@@ -246,7 +264,7 @@ $interpret -bnq use-square-functor.scm
 $compile use-square-functor.scm
 ./a.out
 $compile -s use-square-functor.scm -J
-$interpret -nqe '(import sf1)' -e '(import sf2)'
+$interpret -nqe '(require-library use-square-functor)' -e '(import sf1)' -e '(import sf2)'
 rm -f sf1.import.* sf2.import.* lst.import.* mod.import.*
 
 echo "======================================== compiler syntax tests ..."
@@ -280,9 +298,9 @@ $interpret -s loopy-test.scm
 echo "======================================== r4rstest ..."
 echo "(expect mult-float-print-test to fail)"
 $interpret -e '(set! ##sys#procedure->string (constantly "#<procedure>"))' \
-  -i -s r4rstest.scm >r4rstest.log
+  -i -s r4rstest.scm >r4rstest.out
 
-diff $DIFF_OPTS r4rstest.out r4rstest.log
+diff $DIFF_OPTS r4rstest.expected r4rstest.out
 
 echo "======================================== syntax tests (r5rs_pitfalls) ..."
 echo "(expect two failures)"
@@ -291,14 +309,23 @@ $interpret -i -s r5rs_pitfalls.scm
 echo "======================================== r7rs tests ..."
 $interpret -i -s r7rs-tests.scm
 
-
 echo "======================================== module tests ..."
 $interpret -include-path ${TEST_DIR}/.. -s module-tests.scm
 $interpret -include-path ${TEST_DIR}/.. -s module-tests-2.scm
 
+echo "======================================== module tests (command line options) ..."
+module="test-$(date +%s)"
+$compile test.scm -A -w -j "$module" -module "$module"
+$interpret -e "(import-syntax $module)"
+rm -f "$module.import.scm"
+
 echo "======================================== module tests (compiled) ..."
 $compile module-tests-compiled.scm
 ./a.out
+$compile module-static-eval-compiled.scm
+./a.out
+$compile -static module-static-eval-compiled.scm
+./a.out
 
 echo "======================================== module tests (chained) ..."
 rm -f m*.import.* test-chained-modules.so
@@ -317,8 +344,11 @@ $interpret -bnq ec.so ec-tests.scm
 # $compile ec-tests.scm
 # ./a.out        # takes ages to compile
 
-echo "======================================== hash-table tests ..."
-$interpret -s hash-table-tests.scm
+echo "======================================== module tests (static link) ..."
+$compile_r -static -unit sample-module -J -c sample-module.scm
+mv sample-module.link sample-module.import.scm sample-module.o* "$CHICKEN_INSTALL_REPOSITORY"
+$compile_r -static module-static-link.scm -o a.out
+./a.out
 
 echo "======================================== port tests ..."
 $interpret -s port-tests.scm
@@ -326,30 +356,38 @@ $interpret -s port-tests.scm
 echo "======================================== fixnum tests ..."
 $compile fixnum-tests.scm
 ./a.out
+$compile -unsafe fixnum-tests.scm
+./a.out
+
+echo "======================================== random number tests ..."
+$interpret -s random-tests.scm
 
 echo "======================================== string->number tests ..."
-$compile numbers-string-conversion-tests.scm
+$interpret -s numbers-string-conversion-tests.scm
+$compile -specialize numbers-string-conversion-tests.scm
+./a.out
+
+echo "======================================== basic numeric ops tests ..."
+$interpret -s numbers-test.scm
+$compile -specialize numbers-test.scm
+./a.out
+
+echo "======================================== Alex Shinn's numeric ops tests ..."
+$interpret -s numbers-test-ashinn.scm
+$compile -specialize numbers-test-ashinn.scm
+./a.out
+
+echo "======================================== Gauche's numeric ops tests ..."
+$interpret -s numbers-test-gauche.scm
+$compile -specialize numbers-test-gauche.scm
 ./a.out
 
 echo "======================================== srfi-4 tests ..."
 $interpret -s srfi-4-tests.scm
 
-echo "======================================== srfi-13 tests ..."
-$interpret -s srfi-13-tests.scm
-
-echo "======================================== srfi-14 tests ..."
-$compile srfi-14-tests.scm
-./a.out
-
 echo "======================================== condition tests ..."
 $interpret -s condition-tests.scm
 
-echo "======================================== srfi-18 tests ..."
-$interpret -s simple-thread-test.scm
-$interpret -s mutex-test.scm
-$compile srfi-18-signal-test.scm
-./a.out
-
 echo "======================================== data-structures tests ..."
 $interpret -s data-structures-tests.scm
 
@@ -362,22 +400,20 @@ $interpret -s srfi-45-tests.scm
 echo "======================================== posix tests ..."
 $compile posix-tests.scm
 ./a.out
-rm -fr tmpdir
-mkdir tmpdir
-touch tmpdir/.dotfile
+
+echo "======================================== file access tests ..."
+if test -n "$MSYSTEM"; then
+  $interpret -s file-access-tests.scm //
+  $interpret -s file-access-tests.scm \\
+else
+  $interpret -s file-access-tests.scm /
+fi
 
 echo "======================================== find-files tests ..."
 $interpret -bnq test-find-files.scm
 
-if test -z "$MSYSTEM"; then
-    ln -s /usr tmpdir/symlink
-fi
-
-$interpret -R posix -e '(delete-directory "tmpdir" #t)'
-
-echo "======================================== signal tests ..."
-$compile signal-tests.scm
-./a.out
+echo "======================================== record-renaming tests ..."
+$interpret -bnq record-rename-test.scm
 
 echo "======================================== regular expression tests ..."
 $interpret -bnq test-irregex.scm
@@ -386,7 +422,7 @@ $interpret -bnq test-glob.scm
 echo "======================================== compiler/nursery stress test ..."
 for s in 100000 120000 200000 250000 300000 350000 400000 450000 500000; do
     echo "  $s"
-    ../chicken -ignore-repository ../utils.scm -:s$s -output-file tmp.c -include-path ${TEST_DIR}/..
+    ${CHICKEN} -ignore-repository ../port.scm -:s$s -output-file tmp.c -include-path ${TEST_DIR}/..
 done
 
 echo "======================================== heap literal stress test ..."
@@ -398,8 +434,7 @@ done
 
 echo "======================================== symbol-GC tests ..."
 $compile symbolgc-tests.scm
-# Currently disabled, because this may leave 1 symbol unreclaimed.
-./a.out -:w || echo "*** FAILED ***"
+./a.out
 
 echo "======================================== finalizer tests ..."
 $interpret -s test-finalizers.scm
@@ -418,6 +453,28 @@ $compile locative-stress-test.scm
 echo "======================================== syntax-rules stress test ..."
 $time $interpret -bnq syntax-rule-stress-test.scm
 
+echo "======================================== include test ..."
+mkdir -p a/b
+echo > a/b/ok.scm
+echo '(include "a/b/ok.scm")' > a/b/include.scm
+$compile -analyze-only a/b/include.scm
+echo '(include "b/ok.scm")' > a/b/include.scm
+$compile -analyze-only a/b/include.scm -include-path a
+echo '(include-relative "ok.scm")' > a/b/include.scm
+$compile -analyze-only a/b/include.scm
+echo '(include-relative "b/ok.scm")' > a/include.scm
+$compile -analyze-only a/include.scm
+echo '(include-relative "b/ok.scm")' > a/b/include.scm
+$compile -analyze-only a/b/include.scm -include-path a
+rm -r a
+
+echo "======================================== executable tests ..."
+$compile executable-tests.scm
+./a.out "$TEST_DIR/a.out"
+
+echo "======================================== user pass tests ..."
+$compile -extend user-pass-tests.scm null.scm
+
 echo "======================================== embedding (1) ..."
 $compile embedded1.c
 ./a.out
@@ -430,6 +487,19 @@ echo "======================================== embedding (3) ..."
 $compile -e embedded3.c embedded4.scm
 ./a.out
 
+echo "======================================== linking tests ..."
+$compile_r -unit reverser reverser/tags/1.0/reverser.scm -J -c
+mv reverser/tags/1.0/reverser.o* ./
+$compile_r -link reverser linking-tests.scm -o a.out
+./a.out
+$compile_r -link reverser linking-tests.scm -o a.out -static
+./a.out
+mv reverser.o* reverser.import.scm "$CHICKEN_INSTALL_REPOSITORY"
+$compile_r -link reverser linking-tests.scm -o a.out
+./a.out
+$compile_r -link reverser linking-tests.scm -o a.out -static
+./a.out
+
 echo "======================================== private repository test ..."
 mkdir -p tmp
 $compile private-repository-test.scm -private-repository -o tmp/xxx
@@ -438,35 +508,10 @@ tmp/xxx ${TEST_DIR}/tmp
 PATH=`pwd`/tmp:$PATH xxx ${TEST_DIR}/tmp
 # this may crash, if the PATH contains a non-matching libchicken.dll on Windows:
 #PATH=$PATH:${TEST_DIR}/tmp xxx ${TEST_DIR}/tmp
-rm -fr rev-app rev-app-2 reverser/*.import.* reverser/*.so
 
-echo "======================================== reinstall tests"
-CHICKEN_REPOSITORY=$CHICKEN_REPOSITORY $CHICKEN_UNINSTALL -force reverser
-CHICKEN_REPOSITORY=$CHICKEN_REPOSITORY CSC_OPTIONS=$COMPILE_OPTIONS \
-    CSI_OPTIONS=$SETUP_PREFIX $CHICKEN_INSTALL -t local -l $TEST_DIR \
-    -csi ${TEST_DIR}/../csi reverser:1.0
-CHICKEN_REPOSITORY=$CHICKEN_REPOSITORY $interpret -bnq rev-app.scm 1.0
-CHICKEN_REPOSITORY=$CHICKEN_REPOSITORY CSC_OPTIONS=$COMPILE_OPTIONS \
-    CSI_OPTIONS=$SETUP_PREFIX $CHICKEN_INSTALL -t local -l $TEST_DIR \
-    -reinstall -force -csi ${TEST_DIR}/../csi
-CHICKEN_REPOSITORY=$CHICKEN_REPOSITORY $interpret -bnq rev-app.scm 1.0
-
-if test $OS_NAME != AIX -a $OS_NAME != SunOS -a $OS_NAME != GNU; then
-	echo "======================================== deployment tests"
-	mkdir rev-app
-        TARGET_LIB_PATH=${TEST_DIR}/.. CHICKEN_REPOSITORY=$CHICKEN_REPOSITORY CSC_OPTIONS=$COMPILE_OPTIONS \
-                          CSI_OPTIONS=$SETUP_PREFIX $CHICKEN_INSTALL -csi ${TEST_DIR}/../csi -t local -l $TEST_DIR reverser
-        TARGET_LIB_PATH=${TEST_DIR}/.. CHICKEN_REPOSITORY=$CHICKEN_REPOSITORY CSC_OPTIONS=$COMPILE_OPTIONS \
-                          CSI_OPTIONS=$SETUP_PREFIX $compile2 -deploy rev-app.scm
-        TARGET_LIB_PATH=${TEST_DIR}/.. CHICKEN_REPOSITORY=$CHICKEN_REPOSITORY CSC_OPTIONS=$COMPILE_OPTIONS \
-                          CSI_OPTIONS=$SETUP_PREFIX $CHICKEN_INSTALL -csi ${TEST_DIR}/../csi -deploy -prefix rev-app -t local -l $TEST_DIR reverser
-	unset LD_LIBRARY_PATH DYLD_LIBRARY_PATH CHICKEN_REPOSITORY
-	# An absolute path is required on NetBSD with $ORIGIN, hence `pwd`
-	`pwd`/rev-app/rev-app 1.1
-	mv rev-app rev-app-2
-	`pwd`/rev-app-2/rev-app 1.1
-else
-	echo "Skipping deployment tests: deployment is currently unsupported on your platform."
-fi
+echo "======================================== multiple return values tests ..."
+$interpret -s multiple-values.scm
+$compile multiple-values.scm
+./a.out
 
 echo "======================================== done."
